@@ -4,7 +4,7 @@
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+	   http://www.apache.org/licenses/LICENSE-2.0
 
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,6 +16,7 @@ indent = tab
 tab-size = 4
 */
 
+#include <algorithm>
 #include <csignal>
 #include <clocale>
 #include <pthread.h>
@@ -32,24 +33,29 @@ tab-size = 4
 #include <tuple>
 #include <regex>
 #include <chrono>
+#include <utility>
 #ifdef __APPLE__
 	#include <CoreFoundation/CoreFoundation.h>
 	#include <mach-o/dyld.h>
 	#include <limits.h>
 #endif
+#if !defined(__clang__) && __GNUC__ < 11
+	#include <semaphore.h>
+#else
+	#include <semaphore>
+#endif
 
-#include <btop_shared.hpp>
-#include <btop_tools.hpp>
-#include <btop_config.hpp>
-#include <btop_input.hpp>
-#include <btop_theme.hpp>
-#include <btop_draw.hpp>
-#include <btop_menu.hpp>
+#include "btop_shared.hpp"
+#include "btop_tools.hpp"
+#include "btop_config.hpp"
+#include "btop_input.hpp"
+#include "btop_theme.hpp"
+#include "btop_draw.hpp"
+#include "btop_menu.hpp"
+#include "fmt/core.h"
 
 using std::atomic;
 using std::cout;
-using std::endl;
-using std::endl;
 using std::flush;
 using std::min;
 using std::string;
@@ -58,7 +64,6 @@ using std::to_string;
 using std::vector;
 
 namespace fs = std::filesystem;
-namespace rng = std::ranges;
 
 using namespace Tools;
 using namespace std::chrono_literals;
@@ -73,7 +78,7 @@ namespace Global {
 		{"#801414", "██████╔╝   ██║   ╚██████╔╝██║        ╚═╝    ╚═╝"},
 		{"#000000", "╚═════╝    ╚═╝    ╚═════╝ ╚═╝"},
 	};
-	const string Version = "1.2.13";
+	const string Version = "1.3.0";
 
 	int coreCount;
 	string overlay;
@@ -91,9 +96,9 @@ namespace Global {
 	string exit_error_msg;
 	atomic<bool> thread_exception (false);
 
-    bool debuginit{};   // defaults to false
-    bool debug{};       // defaults to false
-    bool utf_force{};   // defaults to false
+	bool debuginit{};
+	bool debug{};
+	bool utf_force{};
 
 	uint64_t start_time;
 
@@ -102,33 +107,37 @@ namespace Global {
 	atomic<bool> should_quit (false);
 	atomic<bool> should_sleep (false);
 	atomic<bool> _runner_started (false);
+	atomic<bool> init_conf (false);
 
-    bool arg_tty{};         // defaults to false
-    bool arg_low_color{};   // defaults to false
+	bool arg_tty{};
+	bool arg_low_color{};
 	int arg_preset = -1;
+	int arg_update = 0;
 }
 
 //* A simple argument parser
-void argumentParser(const int& argc, char **argv) {
+void argumentParser(const int argc, char **argv) {
 	for(int i = 1; i < argc; i++) {
 		const string argument = argv[i];
 		if (is_in(argument, "-h", "--help")) {
-			cout 	<< "usage: btop [-h] [-v] [-/+t] [-p <id>] [--utf-force] [--debug]\n\n"
-					<< "optional arguments:\n"
-					<< "  -h, --help            show this help message and exit\n"
-					<< "  -v, --version         show version info and exit\n"
-					<< "  -lc, --low-color      disable truecolor, converts 24-bit colors to 256-color\n"
-					<< "  -t, --tty_on          force (ON) tty mode, max 16 colors and tty friendly graph symbols\n"
-					<< "  +t, --tty_off         force (OFF) tty mode\n"
-					<< "  -p, --preset <id>     start with preset, integer value between 0-9\n"
-					<< "  --utf-force           force start even if no UTF-8 locale was detected\n"
-					<< "  --debug               start in DEBUG mode: shows microsecond timer for information collect\n"
-					<< "                        and screen draw functions and sets loglevel to DEBUG\n"
-					<< endl;
+			fmt::println(
+					"usage: btop [-h] [-v] [-/+t] [-p <id>] [-u <ms>] [--utf-force] [--debug]\n\n"
+					"optional arguments:\n"
+					"  -h, --help            show this help message and exit\n"
+					"  -v, --version         show version info and exit\n"
+					"  -lc, --low-color      disable truecolor, converts 24-bit colors to 256-color\n"
+					"  -t, --tty_on          force (ON) tty mode, max 16 colors and tty friendly graph symbols\n"
+					"  +t, --tty_off         force (OFF) tty mode\n"
+					"  -p, --preset <id>     start with preset, integer value between 0-9\n"
+					"  -u, --update <ms>     set the program update rate in milliseconds\n"
+					"  --utf-force           force start even if no UTF-8 locale was detected\n"
+					"  --debug               start in DEBUG mode: shows microsecond timer for information collect\n"
+					"                        and screen draw functions and sets loglevel to DEBUG"
+			);
 			exit(0);
 		}
 		else if (is_in(argument, "-v", "--version")) {
-			cout << "btop version: " << Global::Version << endl;
+			fmt::println("btop version: {}", Global::Version);
 			exit(0);
 		}
 		else if (is_in(argument, "-lc", "--low-color")) {
@@ -144,14 +153,27 @@ void argumentParser(const int& argc, char **argv) {
 		}
 		else if (is_in(argument, "-p", "--preset")) {
 			if (++i >= argc) {
-				cout << "ERROR: Preset option needs an argument." << endl;
+				fmt::println("ERROR: Preset option needs an argument.");
 				exit(1);
 			}
 			else if (const string val = argv[i]; isint(val) and val.size() == 1) {
 				Global::arg_preset = std::clamp(stoi(val), 0, 9);
 			}
 			else {
-				cout << "ERROR: Preset option only accepts an integer value between 0-9." << endl;
+				fmt::println("ERROR: Preset option only accepts an integer value between 0-9.");
+				exit(1);
+			}
+		}
+		else if (is_in(argument, "-u", "--update")) {
+			if (++i >= argc) {
+				fmt::println("ERROR: Update option needs an argument");
+				exit(1);
+			}
+			const std::string value = argv[i];
+			if (isint(value)) {
+				Global::arg_update = std::clamp(std::stoi(value), 100, Config::ONE_DAY_MILLIS);
+			} else {
+				fmt::println("ERROR: Invalid update rate");
 				exit(1);
 			}
 		}
@@ -160,8 +182,8 @@ void argumentParser(const int& argc, char **argv) {
 		else if (argument == "--debug")
 			Global::debug = true;
 		else {
-			cout << " Unknown argument: " << argument << "\n" <<
-			" Use -h or --help for help." <<  endl;
+			fmt::println(" Unknown argument: {}\n"
+				" Use -h or --help for help.", argument);
 			exit(1);
 		}
 	}
@@ -172,7 +194,7 @@ void term_resize(bool force) {
 	static atomic<bool> resizing (false);
 	if (Input::polling) {
 		Global::resized = true;
-		Input::interrupt = true;
+		Input::interrupt();
 		return;
 	}
 	atomic_lock lck(resizing, true);
@@ -180,8 +202,11 @@ void term_resize(bool force) {
 		if (force and refreshed) force = false;
 	}
 	else return;
-
-	static const array<string, 4> all_boxes = {"cpu", "mem", "net", "proc"};
+#ifdef GPU_SUPPORT
+	static const array<string, 10> all_boxes = {"gpu5", "cpu", "mem", "net", "proc", "gpu0", "gpu1", "gpu2", "gpu3", "gpu4"};
+#else
+	static const array<string, 5> all_boxes = {"", "cpu", "mem", "net", "proc"};
+#endif
 	Global::resized = true;
 	if (Runner::active) Runner::stop();
 	Term::refresh();
@@ -189,35 +214,57 @@ void term_resize(bool force) {
 
 	auto boxes = Config::getS("shown_boxes");
 	auto min_size = Term::get_min_size(boxes);
+	auto minWidth = min_size.at(0), minHeight = min_size.at(1);
 
-	while (not force or (Term::width < min_size.at(0) or Term::height < min_size.at(1))) {
+	while (not force or (Term::width < minWidth or Term::height < minHeight)) {
 		sleep_ms(100);
-		if (Term::width < min_size.at(0) or Term::height < min_size.at(1)) {
-			cout << Term::clear << Global::bg_black << Global::fg_white << Mv::to((Term::height / 2) - 2, (Term::width / 2) - 11)
-				 << "Terminal size too small:" << Mv::to((Term::height / 2) - 1, (Term::width / 2) - 10)
-				 << " Width = " << (Term::width < min_size.at(1) ? Global::fg_red : Global::fg_green) << Term::width
-				 << Global::fg_white << " Height = " << (Term::height < min_size.at(0) ? Global::fg_red : Global::fg_green) << Term::height
-				 << Mv::to((Term::height / 2) + 1, (Term::width / 2) - 12) << Global::fg_white
-				 << "Needed for current config:" << Mv::to((Term::height / 2) + 2, (Term::width / 2) - 10)
-				 << "Width = " << min_size.at(0) << " Height = " << min_size.at(1) << flush;
+		if (Term::width < minWidth or Term::height < minHeight) {
+			int width = Term::width, height = Term::height;
+			cout << fmt::format("{clear}{bg_black}{fg_white}"
+					"{mv1}Terminal size too small:"
+					"{mv2} Width = {fg_width}{width} {fg_white}Height = {fg_height}{height}"
+					"{mv3}{fg_white}Needed for current config:"
+					"{mv4}Width = {minWidth} Height = {minHeight}",
+					"clear"_a = Term::clear, "bg_black"_a = Global::bg_black, "fg_white"_a = Global::fg_white,
+					"mv1"_a = Mv::to((height / 2) - 2, (width / 2) - 11),
+					"mv2"_a = Mv::to((height / 2) - 1, (width / 2) - 10),
+						"fg_width"_a = (width < minWidth ? Global::fg_red : Global::fg_green),
+						"width"_a = width,
+						"fg_height"_a = (height < minHeight ? Global::fg_red : Global::fg_green),
+						"height"_a = height,
+					"mv3"_a = Mv::to((height / 2) + 1, (width / 2) - 12),
+					"mv4"_a = Mv::to((height / 2) + 2, (width / 2) - 10),
+						"minWidth"_a = minWidth,
+						"minHeight"_a = minHeight
+			) << std::flush;
+
 			bool got_key = false;
 			for (; not Term::refresh() and not got_key; got_key = Input::poll(10));
 			if (got_key) {
 				auto key = Input::get();
 				if (key == "q")
 					clean_quit(0);
-				else if (is_in(key, "1", "2", "3", "4")) {
-					Config::current_preset = -1;
-					Config::toggle_box(all_boxes.at(std::stoi(key) - 1));
-					boxes = Config::getS("shown_boxes");
+				else if (key.size() == 1 and isint(key)) {
+					auto intKey = stoi(key);
+				#ifdef GPU_SUPPORT
+					if ((intKey == 0 and Gpu::gpu_names.size() >= 5) or (intKey >= 5 and std::cmp_greater_equal(Gpu::gpu_names.size(), intKey - 4))) {
+				#else
+					if (intKey > 0 and intKey < 5) {
+				#endif
+						auto box = all_boxes.at(intKey);
+						Config::current_preset = -1;
+						Config::toggle_box(box);
+						boxes = Config::getS("shown_boxes");
+					}
 				}
 			}
 			min_size = Term::get_min_size(boxes);
+			minWidth = min_size.at(0), minHeight = min_size.at(1);
 		}
 		else if (not Term::refresh()) break;
 	}
 
-	Input::interrupt = true;
+	Input::interrupt();
 }
 
 //* Exit handler; stops threads, restores terminal and saves config changes
@@ -226,21 +273,26 @@ void clean_quit(int sig) {
 	Global::quitting = true;
 	Runner::stop();
 	if (Global::_runner_started) {
-	#ifdef __APPLE__
-		if (pthread_join(Runner::runner_id, NULL) != 0) {
+	#if defined __APPLE__ || defined __OpenBSD__
+		if (pthread_join(Runner::runner_id, nullptr) != 0) {
 			Logger::warning("Failed to join _runner thread on exit!");
 			pthread_cancel(Runner::runner_id);
 		}
 	#else
 		struct timespec ts;
 		ts.tv_sec = 5;
-		if (pthread_timedjoin_np(Runner::runner_id, NULL, &ts) != 0) {
+		if (pthread_timedjoin_np(Runner::runner_id, nullptr, &ts) != 0) {
 			Logger::warning("Failed to join _runner thread on exit!");
 			pthread_cancel(Runner::runner_id);
 		}
 	#endif
 	}
 
+#ifdef GPU_SUPPORT
+	Gpu::Nvml::shutdown();
+	Gpu::Rsmi::shutdown();
+#endif
+
 	Config::write();
 
 	if (Term::initialized) {
@@ -251,13 +303,13 @@ void clean_quit(int sig) {
 	if (not Global::exit_error_msg.empty()) {
 		sig = 1;
 		Logger::error(Global::exit_error_msg);
-		std::cerr << Global::fg_red << "ERROR: " << Global::fg_white << Global::exit_error_msg << Fx::reset << endl;
+		fmt::println(std::cerr, "{}ERROR: {}{}{}", Global::fg_red, Global::fg_white, Global::exit_error_msg, Fx::reset);
 	}
 	Logger::info("Quitting! Runtime: " + sec_to_dhms(time_s() - Global::start_time));
 
 	const auto excode = (sig != -1 ? sig : 0);
 
-#ifdef __APPLE__
+#if defined __APPLE__ || defined __OpenBSD__
 	_Exit(excode);
 #else
 	quick_exit(excode);
@@ -287,7 +339,7 @@ void _signal_handler(const int sig) {
 			if (Runner::active) {
 				Global::should_quit = true;
 				Runner::stopping = true;
-				Input::interrupt = true;
+				Input::interrupt();
 			}
 			else {
 				clean_quit(0);
@@ -297,7 +349,7 @@ void _signal_handler(const int sig) {
 			if (Runner::active) {
 				Global::should_sleep = true;
 				Runner::stopping = true;
-				Input::interrupt = true;
+				Input::interrupt();
 			}
 			else {
 				_sleep();
@@ -309,6 +361,9 @@ void _signal_handler(const int sig) {
 		case SIGWINCH:
 			term_resize();
 			break;
+		case SIGUSR1:
+			// Input::poll interrupt
+			break;
 	}
 }
 
@@ -321,14 +376,12 @@ namespace Runner {
 	atomic<bool> coreNum_reset (false);
 
 	//* Setup semaphore for triggering thread to do work
-#if __GNUC__ < 11
-	#include <semaphore.h>
+#if !defined(__clang__) && __GNUC__ < 11
 	sem_t do_work;
 	inline void thread_sem_init() { sem_init(&do_work, 0, 0); }
 	inline void thread_wait() { sem_wait(&do_work); }
 	inline void thread_trigger() { sem_post(&do_work); }
 #else
-	#include <semaphore>
 	std::binary_semaphore do_work(0);
 	inline void thread_sem_init() { ; }
 	inline void thread_wait() { do_work.acquire(); }
@@ -340,14 +393,14 @@ namespace Runner {
 		pthread_mutex_t& pt_mutex;
 	public:
 		int status;
-        thread_lock(pthread_mutex_t& mtx) : pt_mutex(mtx) {
-            pthread_mutex_init(&pt_mutex, NULL);
-            status = pthread_mutex_lock(&pt_mutex);
-        }
-        ~thread_lock() {
-            if (status == 0)
-                pthread_mutex_unlock(&pt_mutex);
-        }
+		thread_lock(pthread_mutex_t& mtx) : pt_mutex(mtx) {
+			pthread_mutex_init(&pt_mutex, nullptr);
+			status = pthread_mutex_lock(&pt_mutex);
+		}
+		~thread_lock() {
+			if (status == 0)
+				pthread_mutex_unlock(&pt_mutex);
+		}
 	};
 
 	//* Wrapper for raising priviliges when using SUID bit
@@ -355,25 +408,27 @@ namespace Runner {
 		int status = -1;
 	public:
 		gain_priv() {
-            if (Global::real_uid != Global::set_uid)
-                this->status = seteuid(Global::set_uid);
+			if (Global::real_uid != Global::set_uid)
+				this->status = seteuid(Global::set_uid);
 		}
 		~gain_priv() {
-            if (status == 0)
-                status = seteuid(Global::real_uid);
+			if (status == 0)
+				status = seteuid(Global::real_uid);
 		}
 	};
 
 	string output;
 	string empty_bg;
-    bool pause_output{}; // defaults to false
+	bool pause_output{};
 	sigset_t mask;
 	pthread_t runner_id;
 	pthread_mutex_t mtx;
 
 	enum debug_actions {
 		collect_begin,
+		collect_done,
 		draw_begin,
+		draw_begin_only,
 		draw_done
 	};
 
@@ -383,7 +438,15 @@ namespace Runner {
 	};
 
 	string debug_bg;
-	unordered_flat_map<string, array<uint64_t, 2>> debug_times;
+	std::unordered_map<string, array<uint64_t, 2>> debug_times;
+
+	class MyNumPunct : public std::numpunct<char>
+	{
+	protected:
+		virtual char do_thousands_sep() const { return '\''; }
+		virtual std::string do_grouping() const { return "\03"; }
+	};
+
 
 	struct runner_conf {
 		vector<string> boxes;
@@ -401,6 +464,13 @@ namespace Runner {
 			case collect_begin:
 				debug_times[name].at(collect) = time_micros();
 				return;
+			case collect_done:
+				debug_times[name].at(collect) = time_micros() - debug_times[name].at(collect);
+				debug_times["total"].at(collect) += debug_times[name].at(collect);
+				return;
+			case draw_begin_only:
+				debug_times[name].at(draw) = time_micros();
+				return;
 			case draw_begin:
 				debug_times[name].at(draw) = time_micros();
 				debug_times[name].at(collect) = debug_times[name].at(draw) - debug_times[name].at(collect);
@@ -414,21 +484,21 @@ namespace Runner {
 	}
 
 	//? ------------------------------- Secondary thread: async launcher and drawing ----------------------------------
-    void * _runner(void *) {
+	void * _runner(void *) {
 		//? Block some signals in this thread to avoid deadlock from any signal handlers trying to stop this thread
 		sigemptyset(&mask);
 		// sigaddset(&mask, SIGINT);
 		// sigaddset(&mask, SIGTSTP);
 		sigaddset(&mask, SIGWINCH);
 		sigaddset(&mask, SIGTERM);
-		pthread_sigmask(SIG_BLOCK, &mask, NULL);
+		pthread_sigmask(SIG_BLOCK, &mask, nullptr);
 
 		//? pthread_mutex_lock to lock thread and monitor health from main thread
 		thread_lock pt_lck(mtx);
 		if (pt_lck.status != 0) {
 			Global::exit_error_msg = "Exception in runner thread -> pthread_mutex_lock error id: " + to_string(pt_lck.status);
 			Global::thread_exception = true;
-			Input::interrupt = true;
+			Input::interrupt();
 			stopping = true;
 		}
 
@@ -439,7 +509,7 @@ namespace Runner {
 			if (active) {
 				Global::exit_error_msg = "Runner thread failed to get active lock!";
 				Global::thread_exception = true;
-				Input::interrupt = true;
+				Input::interrupt();
 				stopping = true;
 			}
 			if (stopping or Global::resized) {
@@ -458,7 +528,13 @@ namespace Runner {
 			//! DEBUG stats
 			if (Global::debug) {
                 if (debug_bg.empty() or redraw)
-                    Runner::debug_bg = Draw::createBox(2, 2, 32, 8, "", true, "debug");
+                    Runner::debug_bg = Draw::createBox(2, 2, 33,
+					#ifdef GPU_SUPPORT
+						9,
+					#else
+						8,
+					#endif
+					"", true, "μs");
 
 				debug_times.clear();
 				debug_times["total"] = {0, 0};
@@ -468,6 +544,29 @@ namespace Runner {
 
 			//* Run collection and draw functions for all boxes
 			try {
+			#ifdef GPU_SUPPORT
+				//? GPU data collection
+				const bool gpu_in_cpu_panel = Gpu::gpu_names.size() > 0 and (
+					Config::getS("cpu_graph_lower").starts_with("gpu-") or Config::getS("cpu_graph_upper").starts_with("gpu-")
+					or (Gpu::shown == 0 and Config::getS("show_gpu_info") != "Off")
+				);
+
+				vector<unsigned int> gpu_panels = {};
+				for (auto& box : conf.boxes)
+					if (box.starts_with("gpu"))
+						gpu_panels.push_back(box.back()-'0');
+
+				vector<Gpu::gpu_info> gpus;
+				if (gpu_in_cpu_panel or not gpu_panels.empty()) {
+					if (Global::debug) debug_timer("gpu", collect_begin);
+					gpus = Gpu::collect(conf.no_update);
+					if (Global::debug) debug_timer("gpu", collect_done);
+				}
+				auto& gpus_ref = gpus;
+			#else
+				vector<Gpu::gpu_info> gpus_ref{};
+			#endif
+
 				//? CPU
 				if (v_contains(conf.boxes, "cpu")) {
 					try {
@@ -480,22 +579,39 @@ namespace Runner {
 							coreNum_reset = false;
 							Cpu::core_mapping = Cpu::get_core_mapping();
 							Global::resized = true;
-							Input::interrupt = true;
+							Input::interrupt();
 							continue;
 						}
 
 						if (Global::debug) debug_timer("cpu", draw_begin);
 
 						//? Draw box
-						if (not pause_output) output += Cpu::draw(cpu, conf.force_redraw, conf.no_update);
+						if (not pause_output) output += Cpu::draw(cpu, gpus_ref, conf.force_redraw, conf.no_update);
 
 						if (Global::debug) debug_timer("cpu", draw_done);
 					}
 					catch (const std::exception& e) {
-                        throw std::runtime_error("Cpu:: -> " + string{e.what()});
+						throw std::runtime_error("Cpu:: -> " + string{e.what()});
 					}
 				}
+			#ifdef GPU_SUPPORT
+				//? GPU
+				if (not gpu_panels.empty() and not gpus_ref.empty()) {
+					try {
+						if (Global::debug) debug_timer("gpu", draw_begin_only);
 
+						//? Draw box
+						if (not pause_output)
+							for (unsigned long i = 0; i < gpu_panels.size(); ++i)
+								output += Gpu::draw(gpus_ref[gpu_panels[i]], i, conf.force_redraw, conf.no_update);
+
+						if (Global::debug) debug_timer("gpu", draw_done);
+					}
+					catch (const std::exception& e) {
+                        throw std::runtime_error("Gpu:: -> " + string{e.what()});
+					}
+				}
+			#endif
 				//? MEM
 				if (v_contains(conf.boxes, "mem")) {
 					try {
@@ -512,7 +628,7 @@ namespace Runner {
 						if (Global::debug) debug_timer("mem", draw_done);
 					}
 					catch (const std::exception& e) {
-                        throw std::runtime_error("Mem:: -> " + string{e.what()});
+						throw std::runtime_error("Mem:: -> " + string{e.what()});
 					}
 				}
 
@@ -532,7 +648,7 @@ namespace Runner {
 						if (Global::debug) debug_timer("net", draw_done);
 					}
 					catch (const std::exception& e) {
-                        throw std::runtime_error("Net:: -> " + string{e.what()});
+						throw std::runtime_error("Net:: -> " + string{e.what()});
 					}
 				}
 
@@ -552,14 +668,15 @@ namespace Runner {
 						if (Global::debug) debug_timer("proc", draw_done);
 					}
 					catch (const std::exception& e) {
-                        throw std::runtime_error("Proc:: -> " + string{e.what()});
+						throw std::runtime_error("Proc:: -> " + string{e.what()});
 					}
 				}
+
 			}
 			catch (const std::exception& e) {
-                Global::exit_error_msg = "Exception in runner thread -> " + string{e.what()};
+				Global::exit_error_msg = "Exception in runner thread -> " + string{e.what()};
 				Global::thread_exception = true;
-				Input::interrupt = true;
+				Input::interrupt();
 				stopping = true;
 			}
 
@@ -578,26 +695,53 @@ namespace Runner {
 				if (empty_bg.empty()) {
 					const int x = Term::width / 2 - 10, y = Term::height / 2 - 10;
 					output += Term::clear;
-					empty_bg += Draw::banner_gen(y, 0, true)
-						+ Mv::to(y+6, x) + Theme::c("title") + Fx::b + "No boxes shown!"
-						+ Mv::to(y+8, x) + Theme::c("hi_fg") + "1" + Theme::c("main_fg") + " | Show CPU box"
-						+ Mv::to(y+9, x) + Theme::c("hi_fg") + "2" + Theme::c("main_fg") + " | Show MEM box"
-						+ Mv::to(y+10, x) + Theme::c("hi_fg") + "3" + Theme::c("main_fg") + " | Show NET box"
-						+ Mv::to(y+11, x) + Theme::c("hi_fg") + "4" + Theme::c("main_fg") + " | Show PROC box"
-						+ Mv::to(y+12, x-2) + Theme::c("hi_fg") + "esc" + Theme::c("main_fg") + " | Show menu"
-						+ Mv::to(y+13, x) + Theme::c("hi_fg") + "q" + Theme::c("main_fg") + " | Quit";
+					empty_bg = fmt::format(
+						"{banner}"
+						"{mv1}{titleFg}{b}No boxes shown!"
+						"{mv2}{hiFg}1 {mainFg}| Show CPU box"
+						"{mv3}{hiFg}2 {mainFg}| Show MEM box"
+						"{mv4}{hiFg}3 {mainFg}| Show NET box"
+						"{mv5}{hiFg}4 {mainFg}| Show PROC box"
+						"{mv6}{hiFg}5-0 {mainFg}| Show GPU boxes"
+						"{mv7}{hiFg}esc {mainFg}| Show menu"
+						"{mv8}{hiFg}q {mainFg}| Quit",
+						"banner"_a = Draw::banner_gen(y, 0, true),
+						"titleFg"_a = Theme::c("title"), "b"_a = Fx::b, "hiFg"_a = Theme::c("hi_fg"), "mainFg"_a = Theme::c("main_fg"),
+						"mv1"_a = Mv::to(y+6, x),
+						"mv2"_a = Mv::to(y+8, x),
+						"mv3"_a = Mv::to(y+9, x),
+						"mv4"_a = Mv::to(y+10, x),
+						"mv5"_a = Mv::to(y+11, x),
+						"mv6"_a = Mv::to(y+12, x-2),
+						"mv7"_a = Mv::to(y+13, x-2),
+						"mv8"_a = Mv::to(y+14, x)
+					);
 				}
 				output += empty_bg;
 			}
 
 			//! DEBUG stats -->
 			if (Global::debug and not Menu::active) {
-				output += debug_bg + Theme::c("title") + Fx::b + ljust(" Box", 9) + ljust("Collect μs", 12, true) + ljust("Draw μs", 9, true) + Theme::c("main_fg") + Fx::ub;
+				output += fmt::format("{pre}{box:5.5} {collect:>12.12} {draw:>12.12}{post}",
+					"pre"_a = debug_bg + Theme::c("title") + Fx::b,
+					"box"_a = "box", "collect"_a = "collect", "draw"_a = "draw",
+					"post"_a = Theme::c("main_fg") + Fx::ub
+				);
+				static auto loc = std::locale(std::locale::classic(), new MyNumPunct);
+			#ifdef GPU_SUPPORT
+				for (const string name : {"cpu", "mem", "net", "proc", "gpu", "total"}) {
+			#else
 				for (const string name : {"cpu", "mem", "net", "proc", "total"}) {
+			#endif
 					if (not debug_times.contains(name)) debug_times[name] = {0,0};
 					const auto& [time_collect, time_draw] = debug_times.at(name);
 					if (name == "total") output += Fx::b;
-					output += Mv::l(29) + Mv::d(1) + ljust(name, 8) + ljust(to_string(time_collect), 12) + ljust(to_string(time_draw), 9);
+					output += fmt::format(loc, "{mvLD}{name:5.5} {collect:12L} {draw:12L}",
+						"mvLD"_a = Mv::l(31) + Mv::d(1),
+						"name"_a = name,
+						"collect"_a = time_collect,
+						"draw"_a = time_draw
+					);
 				}
 			}
 
@@ -608,19 +752,19 @@ namespace Runner {
 				<< Term::sync_end << flush;
 		}
 		//* ----------------------------------------------- THREAD LOOP -----------------------------------------------
-		pthread_exit(NULL);
+		return {};
 	}
 	//? ------------------------------------------ Secondary thread end -----------------------------------------------
 
 	//* Runs collect and draw in a secondary thread, unlocks and locks config to update cached values
-    void run(const string& box, bool no_update, bool force_redraw) {
+	void run(const string& box, bool no_update, bool force_redraw) {
 		atomic_wait_for(active, true, 5000);
 		if (active) {
 			Logger::error("Stall in Runner thread, restarting!");
 			active = false;
 			// exit(1);
 			pthread_cancel(Runner::runner_id);
-			if (pthread_create(&Runner::runner_id, NULL, &Runner::_runner, NULL) != 0) {
+			if (pthread_create(&Runner::runner_id, nullptr, &Runner::_runner, nullptr) != 0) {
 				Global::exit_error_msg = "Failed to re-create _runner thread!";
 				clean_quit(1);
 			}
@@ -706,29 +850,23 @@ int main(int argc, char **argv) {
 	//? Call argument parser if launched with arguments
 	if (argc > 1) argumentParser(argc, argv);
 
-	//? Setup paths for config, log and user themes
-	for (const auto& env : {"XDG_CONFIG_HOME", "HOME"}) {
-		if (std::getenv(env) != NULL and access(std::getenv(env), W_OK) != -1) {
-			Config::conf_dir = fs::path(std::getenv(env)) / (((string)env == "HOME") ? ".config/btop" : "btop");
-			break;
-		}
-	}
-	if (Config::conf_dir.empty()) {
-		cout 	<< "WARNING: Could not get path user HOME folder.\n"
-				<< "Make sure $XDG_CONFIG_HOME or $HOME environment variables is correctly set to fix this." << endl;
-	}
-	else {
-		if (std::error_code ec; not fs::is_directory(Config::conf_dir) and not fs::create_directories(Config::conf_dir, ec)) {
-			cout 	<< "WARNING: Could not create or access btop config directory. Logging and config saving disabled.\n"
-					<< "Make sure $XDG_CONFIG_HOME or $HOME environment variables is correctly set to fix this." << endl;
-		}
-		else {
+	{
+		const auto config_dir = Config::get_config_dir();
+		if (config_dir.has_value()) {
+			Config::conf_dir = config_dir.value();
 			Config::conf_file = Config::conf_dir / "btop.conf";
 			Logger::logfile = Config::conf_dir / "btop.log";
 			Theme::user_theme_dir = Config::conf_dir / "themes";
-			if (not fs::exists(Theme::user_theme_dir) and not fs::create_directory(Theme::user_theme_dir, ec)) Theme::user_theme_dir.clear();
+
+			// If necessary create the user theme directory
+			std::error_code error;
+			if (not fs::exists(Theme::user_theme_dir, error) and not fs::create_directories(Theme::user_theme_dir, error)) {
+				Theme::user_theme_dir.clear();
+				Logger::warning("Failed to create user theme directory: " + error.message());
+			}
 		}
 	}
+
 	//? Try to find global btop theme path relative to binary path
 #ifdef __linux__
 	{ 	std::error_code ec;
@@ -757,10 +895,10 @@ int main(int argc, char **argv) {
 	}
 
 	//? Config init
-	{	vector<string> load_warnings;
+	{
+		atomic_lock lck(Global::init_conf);
+		vector<string> load_warnings;
 		Config::load(Config::conf_file, load_warnings);
-
-		if (Config::current_boxes.empty()) Config::check_boxes(Config::getS("shown_boxes"));
 		Config::set("lowcolor", (Global::arg_low_color ? true : not Config::getB("truecolor")));
 
 		if (Global::debug) {
@@ -775,17 +913,17 @@ int main(int argc, char **argv) {
 	}
 
 	//? Try to find and set a UTF-8 locale
-	if (std::setlocale(LC_ALL, "") != NULL and not s_contains((string)std::setlocale(LC_ALL, ""), ";")
+	if (std::setlocale(LC_ALL, "") != nullptr and not s_contains((string)std::setlocale(LC_ALL, ""), ";")
 	and str_to_upper(s_replace((string)std::setlocale(LC_ALL, ""), "-", "")).ends_with("UTF8")) {
 		Logger::debug("Using locale " + (string)std::setlocale(LC_ALL, ""));
 	}
 	else {
 		string found;
-        bool set_failure{}; // defaults to false
+		bool set_failure{};
 		for (const auto loc_env : array{"LANG", "LC_ALL"}) {
-			if (std::getenv(loc_env) != NULL and str_to_upper(s_replace((string)std::getenv(loc_env), "-", "")).ends_with("UTF8")) {
+			if (std::getenv(loc_env) != nullptr and str_to_upper(s_replace((string)std::getenv(loc_env), "-", "")).ends_with("UTF8")) {
 				found = std::getenv(loc_env);
-				if (std::setlocale(LC_ALL, found.c_str()) == NULL) {
+				if (std::setlocale(LC_ALL, found.c_str()) == nullptr) {
 					set_failure = true;
 					Logger::warning("Failed to set locale " + found + " continuing anyway.");
 				}
@@ -798,7 +936,7 @@ int main(int argc, char **argv) {
 						for (auto& l : ssplit(loc, ';')) {
 							if (str_to_upper(s_replace(l, "-", "")).ends_with("UTF8")) {
 								found = l.substr(l.find('=') + 1);
-								if (std::setlocale(LC_ALL, found.c_str()) != NULL) {
+								if (std::setlocale(LC_ALL, found.c_str()) != nullptr) {
 									break;
 								}
 							}
@@ -808,7 +946,7 @@ int main(int argc, char **argv) {
 				catch (...) { found.clear(); }
 			}
 		}
-
+	//
 	#ifdef __APPLE__
 		if (found.empty()) {
 			CFLocaleRef cflocale = CFLocaleCopyCurrent();
@@ -819,10 +957,10 @@ int main(int argc, char **argv) {
 			if (cur_locale.empty()) {
 				Logger::warning("No UTF-8 locale detected! Some symbols might not display correctly.");
 			}
-			else if (std::setlocale(LC_ALL, string(cur_locale + ".UTF-8").c_str()) != NULL) {
+			else if (std::setlocale(LC_ALL, string(cur_locale + ".UTF-8").c_str()) != nullptr) {
 				Logger::debug("Setting LC_ALL=" + cur_locale + ".UTF-8");
 			}
-			else if(std::setlocale(LC_ALL, "en_US.UTF-8") != NULL) {
+			else if(std::setlocale(LC_ALL, "en_US.UTF-8") != nullptr) {
 				Logger::debug("Setting LC_ALL=en_US.UTF-8");
 			}
 			else {
@@ -852,7 +990,7 @@ int main(int argc, char **argv) {
 		Config::set("tty_mode", true);
 		Logger::info("Forcing tty mode: setting 16 color mode and using tty friendly graph symbols");
 	}
-#ifndef __APPLE__
+#if not defined __APPLE__ && not defined __OpenBSD__
 	else if (not Global::arg_tty and Term::current_tty.starts_with("/dev/tty")) {
 		Config::set("tty_mode", true);
 		Logger::info("Real tty detected: setting 16 color mode and using tty friendly graph symbols");
@@ -877,10 +1015,15 @@ int main(int argc, char **argv) {
 		Shared::init();
 	}
 	catch (const std::exception& e) {
-        Global::exit_error_msg = "Exception in Shared::init() -> " + string{e.what()};
+		Global::exit_error_msg = "Exception in Shared::init() -> " + string{e.what()};
 		clean_quit(1);
 	}
 
+	if (not Config::check_boxes(Config::getS("shown_boxes"))) {
+		Config::check_boxes("cpu mem net proc");
+		Config::set("shown_boxes", "cpu mem net proc"s);
+	}
+
 	//? Update list of available themes and generate the selected theme
 	Theme::updateThemes();
 	Theme::setTheme();
@@ -891,10 +1034,16 @@ int main(int argc, char **argv) {
 	std::signal(SIGTSTP, _signal_handler);
 	std::signal(SIGCONT, _signal_handler);
 	std::signal(SIGWINCH, _signal_handler);
+	std::signal(SIGUSR1, _signal_handler);
+
+	sigset_t mask;
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGUSR1);
+	pthread_sigmask(SIG_BLOCK, &mask, &Input::signal_mask);
 
 	//? Start runner thread
 	Runner::thread_sem_init();
-	if (pthread_create(&Runner::runner_id, NULL, &Runner::_runner, NULL) != 0) {
+	if (pthread_create(&Runner::runner_id, nullptr, &Runner::_runner, nullptr) != 0) {
 		Global::exit_error_msg = "Failed to create _runner thread!";
 		clean_quit(1);
 	}
@@ -912,9 +1061,10 @@ int main(int argc, char **argv) {
 	{
 		const auto [x, y] = Term::get_min_size(Config::getS("shown_boxes"));
 		if (Term::height < y or Term::width < x) {
+			pthread_sigmask(SIG_SETMASK, &Input::signal_mask, &mask);
 			term_resize(true);
+			pthread_sigmask(SIG_SETMASK, &mask, nullptr);
 			Global::resized = false;
-			Input::interrupt = false;
 		}
 
 	}
@@ -927,6 +1077,9 @@ int main(int argc, char **argv) {
 
 	//? ------------------------------------------------ MAIN LOOP ----------------------------------------------------
 
+	if (Global::arg_update != 0) {
+		Config::set("update_ms", Global::arg_update);
+	}
 	uint64_t update_ms = Config::getI("update_ms");
 	auto future_time = time_ms();
 
@@ -989,7 +1142,7 @@ int main(int argc, char **argv) {
 		}
 	}
 	catch (const std::exception& e) {
-        Global::exit_error_msg = "Exception in main loop -> " + string{e.what()};
+		Global::exit_error_msg = "Exception in main loop -> " + string{e.what()};
 		clean_quit(1);
 	}
 
