@@ -4,7 +4,7 @@
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+	   http://www.apache.org/licenses/LICENSE-2.0
 
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,18 +21,17 @@ tab-size = 4
 #include <string>
 #include <atomic>
 #include <array>
-#include <robin_hood.h>
+#include <unordered_map>
 #include <deque>
 
-using robin_hood::unordered_flat_map;
 using std::array;
 using std::atomic;
 using std::deque;
 using std::string;
 
-/* The input functions relies on the following std::cin options being set:
-	cin.sync_with_stdio(false);
-	cin.tie(NULL);
+/* The input functions rely on the following termios parameters being set:
+	Non-canonical mode (c_lflags & ~(ICANON))
+	VMIN and VTIME (c_cc) set to 0
 	These will automatically be set when running Term::init() from btop_tools.cpp
 */
 
@@ -44,9 +43,11 @@ namespace Input {
 	};
 
 	//? line, col, height, width
-	extern unordered_flat_map<string, Mouse_loc> mouse_mappings;
+	extern std::unordered_map<string, Mouse_loc> mouse_mappings;
+
+	//* Signal mask used during polling read
+	extern sigset_t signal_mask;
 
-	extern atomic<bool> interrupt;
 	extern atomic<bool> polling;
 
 	//* Mouse column and line position
@@ -56,7 +57,7 @@ namespace Input {
 	extern deque<string> history;
 
 	//* Poll keyboard & mouse input for <timeout> ms and return input availabilty as a bool
-	bool poll(int timeout=0);
+	bool poll(const uint64_t timeout=0);
 
 	//* Get a key or mouse action from input
 	string get();
@@ -64,6 +65,9 @@ namespace Input {
 	//* Wait until input is available and return key
 	string wait();
 
+	//* Interrupt poll/wait
+	void interrupt();
+
 	//* Clears last entered key
 	void clear();
 
